.. Berechnet die Rendite eines Wertpapieres

function rendite (x)
## berechnet die Rendite von Zahlungen x zu Perioden 0,1,2,3,...
    if sum(x)<0; error("Rendite negativ?"); endif;
    q=polydif(x);
    c=1;
    repeat
        cnew=c-polyval(x,c)/polyval(q,c);
        if c~=cnew; break; endif;
        c=cnew;
    end;
    return (1/cnew-1)*100;
endfunction

function rendite2 (x,n)
## berechnet die Rendite von Zahlungen x zu gebrochenen Perioden n.
    if sum(x)<0; error("Rendite negativ?"); endif;
    k2=x*n; n2=n-1;
    k2=k2[2:length(x)]; n2=n2[2:length(x)];
    c=1;
    repeat
        cnew=c-sum(x*c^n)/sum(k2*c^n2);
        if c~=cnew; break; endif;
        c=cnew;
    end;
    return (1/cnew-1)*100;
endfunction

function effzins (kurs,laufzeit,zins)
## berechnet den effektiven Zins eines Wertpapieres.
## kurs und zins in %, laufzeit in Zinsperioden.
## z.B. effzins(104,8,8.75).
    si=size(kurs,laufzeit,zins); rend=zeros(si);
    loop 1 to prod(si);
        k=-kurs{#}|dup(zins{#},laufzeit{#})';
        k[laufzeit{#}+1]=k[laufzeit{#}+1]+100;
        rend{#}=rendite(k);
    end;
    return rend
endfunction

function tilgung (darlehen,laufzeit,zins)
## berechnet die Tilgung eines Darlehens.
    p=1+zins/100;
    return p*(1/p-1)*darlehen/(1/p^laufzeit-1);
endfunction

function darlzins (darlehen,laufzeit,tilgung)
## berechnet den Zinssatz eines Darlehens.
    si=size(darlehen,laufzeit,tilgung); z=zeros(si);
    loop 1 to prod(si);
        k=-darlehen{#}|dup(tilgung{#},laufzeit{#})';
        z{#}=rendite(k);
    end;
    return z
endfunction

function sparzins (rate,laufzeit,endkapital)
## berechnet den effektiven Zins eines Sparvertrages.
    si=size(rate,laufzeit,endkapital); z=zeros(si);
    loop 1 to prod(si);
        k=dup(-rate{#},laufzeit{#})'|endkapital{#};
        z{#}=rendite(k);
    end;
    return z
endfunction

function endkap (rate,laufzeit,zins)
## berechnet das Endkapital eines Sparvertrages.
	p=1+zins/100;
    return p*rate*(p^laufzeit-1)/(p-1);
endfunction

"rendite(R) definiert."
"rendite2(T,R) definiert."
"effzins(W,T,Z) definiert."
"tilgung(W,T,Z) definiert."
"darlzins(W,T,R) definiert."
"sparzins(R,T,W) definiert."
"endkap(R,T,Z) definiert."

"Benutzen Sie help ... fr weitere Informationen."

