
/**************************************
 * RESOURCE.C                         *
 **************************************
 * Ntzliche GEM-Routinen fr Radio-  *
 * buttons, Checkboxen etc.           *
 **************************************
 * Hier f. Projekt "Schoner"          *
 **************************************
 * Stammt von Dirk Sabiwalsky. Thanx! *
 **************************************
 * Pure-C, Tab: 5 (!), Atari          *
 **************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <vdi.h>
#include <aes.h>

#include "schoner.h"          /* RSC a la "RSC2CSRC" decompiliert     */
#include "schoner.rsh"        /* (Gibt's z.B. bei MAXON)              */

#define TRUE   1
#define FALSE  0

/* Fontgren */
#define T_NORMAL    2
#define T_MEDIUM    1
#define T_SMALL     0

typedef struct      /* Ein Button... */
{
     int on[16];
     int off[16];
} EASE_SWITCH;

#include "buttons.h"

int  pxy[8],clipx1,clipy1,clipx2,clipy2;
MFDB mf_source,mf_dest;
USERBLK user_bmac,user_rmac,user_text;

extern int virt_handle,ch,cb;
extern OBJECT rs_object[];

void fix_pixels(int *coord,int this_size, int design_size)
{
     int px;
     px = *coord >> 8;
     if (px)
     {
          px = px * this_size/design_size;
          *coord = (*coord & 0xff) | (px << 8);
     }
}

void fix_koordinates(void)
{
     int i;
     OBJECT *this_ob;

     for(i = 0; i < NUM_OBS; i++)
     {
          this_ob = &rs_object[i];

          if(   this_ob >= TREE[1]
             && (this_ob->ob_type & 0xff) != G_IMAGE
             && (this_ob->ob_type & 0xff) != G_ICON)
          {
               fix_pixels(&(this_ob->ob_x),cb,8);
               fix_pixels(&(this_ob->ob_y),ch,16);
               fix_pixels(&(this_ob->ob_width),cb,8);
               fix_pixels(&(this_ob->ob_height),ch,16);
          }
     }
}

/* Clipping-Rechteck setzen */

void set_clipping(int clip, int clx1, int cly1, int clx2, int cly2)
{
     pxy[0]=clipx1=clx1;
     pxy[1]=clipy1=cly1;
     pxy[2]=clipx2=clx2;
     pxy[3]=clipy2=cly2;
     vs_clip(virt_handle,clip,pxy);
}

/* einen MFDB vorbelegen */

void set_mfdb(MFDB *mfdb, void *data, int width, int height, int words, int stand, int planes)
{
     mfdb->fd_addr=data;
     mfdb->fd_w=width;
     mfdb->fd_h=height;
     mfdb->fd_wdwidth=words;
     mfdb->fd_stand=stand;
     mfdb->fd_nplanes=planes;
}

/* ein 16x16-Pixel Icon zeichnen */

void draw_smallicon(char *icon, int mode, int x, int y)
{
     int  color[2];

     color[0]=1;
     color[1]=0;

     set_mfdb(&mf_source,icon,16,16,1,1,1);

     mf_dest.fd_addr=NULL;

     pxy[0]=0;
     pxy[1]=0;
     pxy[2]=15;
     pxy[3]=15;
     pxy[4]=x;
     pxy[5]=y;
     pxy[6]=x+15;
     pxy[7]=y+15;

     vrt_cpyfm(virt_handle,mode,pxy,&mf_source,&mf_dest,color);
}

/* ein 8*8-Icon aus einer 16*8-Bitmap zeichnen */

void draw_midicon(int *icon, int mode, int x, int y)
{
     int  color[2];

     color[0]=1;
     color[1]=0;

     set_mfdb(&mf_source,(char*)icon,16,8,1,1,1);

     mf_dest.fd_addr=NULL;

     pxy[0]=0;
     pxy[1]=0;
     pxy[2]=7;
     pxy[3]=7;
     pxy[4]=x;
     pxy[5]=y;
     pxy[6]=x+7;
     pxy[7]=y+7;

     vrt_cpyfm(virt_handle,mode,pxy,&mf_source,&mf_dest,color);
}

/* Text zeichnen */

void print_text(int x, int y, char *s)
{
     v_gtext(virt_handle,x,y,s);
}

/* Font setzen */

void set_font(int font)
{
     int  dummy;

     switch(font)
     {
          case T_SMALL:  vst_height(virt_handle,4,&dummy,&dummy,&dummy,&dummy);
                         break;
          case T_MEDIUM: vst_height(virt_handle,6,&dummy,&dummy,&dummy,&dummy);
                         break;
          case T_NORMAL: if(ch==16)
                           vst_height(virt_handle,13,&dummy,&dummy,&dummy,&dummy);
                         else
                           vst_height(virt_handle,6,&dummy,&dummy,&dummy,&dummy);
                         break;
     }
}

/* Linie zeichnen */

void draw_line(int x1, int y1, int x2, int y2)
{
     pxy[0]=x1;
     pxy[1]=y1;
     pxy[2]=x2;
     pxy[3]=y2;
     v_pline(virt_handle,2,pxy);
}

/* Checkbox zeichnen */

int cdecl draw_bmac(PARMBLK *pb)
{
     int  x,y;

     x=pb->pb_x;
     y=pb->pb_y;

     set_clipping(TRUE,pb->pb_xc,pb->pb_yc,pb->pb_xc+pb->pb_wc-1,pb->pb_yc+pb->pb_hc-1);

     if(ch==16)
       draw_smallicon((pb->pb_currstate & SELECTED) ? (char*)cross_button.on : (char*)cross_button.off,
                       MD_REPLACE,x,y);
     else
       draw_midicon((pb->pb_currstate & SELECTED) ? (int*)mcross_button.on : (int*)mcross_button.off,
                     MD_REPLACE,x,y);

     set_font(T_NORMAL);
     vswr_mode(virt_handle,MD_TRANS);
     print_text(x+24,y,(char*)pb->pb_parm);
     vswr_mode(virt_handle,MD_REPLACE);

     return(pb->pb_currstate & ~SELECTED);
}

/* Radiobutton zeichnen */

int cdecl draw_rmac(PARMBLK *pb)
{
     int  x,y;

     x=pb->pb_x;
     y=pb->pb_y;

     set_clipping(TRUE,pb->pb_xc,pb->pb_yc,pb->pb_xc+pb->pb_wc-1,pb->pb_yc+pb->pb_hc-1);

     if(ch==16)
       draw_smallicon((pb->pb_currstate & SELECTED) ? (char*)radio_button.on : (char*)radio_button.off,
                       MD_REPLACE,x,y);
     else
       draw_midicon((pb->pb_currstate & SELECTED) ? (int*)mradio_button.on : (int*)mradio_button.off,
                     MD_REPLACE,x,y);

     set_font(T_NORMAL);
     vswr_mode(virt_handle,MD_TRANS);
     print_text(x+24,y,(char*)pb->pb_parm);
     vswr_mode(virt_handle,MD_REPLACE);

     return(pb->pb_currstate & ~SELECTED);
}

/* Unterstrichenen Text zeichnen */

int cdecl draw_underlined(PARMBLK *pb)
{
     int  x,y,pxy[4];

     x=pb->pb_x-1 + (pb->pb_w - cb * (int)strlen((char*)pb->pb_parm) ) / 2;
     y=pb->pb_y-1;

     pxy[0]=pb->pb_xc;
     pxy[1]=pb->pb_yc;
     pxy[2]=pb->pb_xc+pb->pb_wc-1;
     pxy[3]=pb->pb_yc+pb->pb_hc-1;
     vs_clip(virt_handle,1,pxy);

     vst_color(virt_handle,2);
     print_text(x,y,(char*)pb->pb_parm);
     vst_color(virt_handle,1);
     draw_line(x,y+ch-1,x+(int)strlen((char*)pb->pb_parm)*cb,y+ch-1);

     return(pb->pb_currstate & ~SELECTED);
}

/* Userdef-Objekte initialisieren */

void init_userdefs(void)
{
     USERBLK   *ub;
     unsigned  i;

     user_bmac.ub_code=draw_bmac;
     user_bmac.ub_parm=0L;

     user_rmac.ub_code=draw_rmac;
     user_rmac.ub_parm=0L;

     user_text.ub_code=draw_underlined;
     user_text.ub_parm=0L;

     for(i=0;i<NUM_OBS;i++)
     {
          switch(rs_object[i].ob_type & 0xff)
          {
               case G_TEXT    : /* Unterstrichen ? */
                                if(rs_object[i].ob_flags & 1024) /* Flags 10 ??? */
                                  if((ub=malloc(sizeof(USERBLK)))!=NULL)
                                  {
                                        memcpy(ub,&user_text,sizeof(USERBLK));
                                        rs_object[i].ob_type=G_USERDEF;
                                        ub->ub_parm=(long)(rs_object[i].ob_spec.tedinfo->te_ptext);
                                        rs_object[i].ob_spec.userblk=ub;
                                  }
                                break;
               case G_BUTTON  : if(!(rs_object[i].ob_flags & EXIT))
                                  if(rs_object[i].ob_flags & SELECTABLE)
                                    if(rs_object[i].ob_flags & RBUTTON)
                                      if((ub=malloc(sizeof(USERBLK)))!=NULL)
                                      {
                                        rs_object[i].ob_width+=24;
                                        memcpy(ub,&user_rmac,sizeof(USERBLK));
                                        rs_object[i].ob_type=G_USERDEF;
                                        ub->ub_parm=(long)(rs_object[i].ob_spec.free_string);
                                        rs_object[i].ob_spec.userblk=ub;
                                      }
                                      else;
                                    else
                                      if((ub=malloc(sizeof(USERBLK)))!=NULL)
                                      {
                                        rs_object[i].ob_width+=24;
                                        memcpy(ub,&user_bmac,sizeof(USERBLK));
                                        rs_object[i].ob_type=G_USERDEF;
                                        ub->ub_parm=(long)(rs_object[i].ob_spec.free_string);
                                        rs_object[i].ob_spec.userblk=ub;
                                      }
                                break;
          }
          /* 1/2 Zeile hochsetzen ? */
          if(rs_object[i].ob_flags & 2048) /* Flags 11 ??? */
          {
               rs_object[i].ob_flags &= !2048;
               rs_object[i].ob_y += ch/2;
          }
     }
}

/* Speicher fr Userdef-Objekte freigeben */

void forget_userdefs(void)
{
     unsigned  i;

     for(i=0;i<NUM_OBS;i++)
       if(   rs_object[i].ob_type==G_USERDEF
          && (   rs_object[i].ob_spec.userblk->ub_code==draw_bmac
              || rs_object[i].ob_spec.userblk->ub_code==draw_rmac))
         free(rs_object[i].ob_spec.userblk);
}

/***********************
 * Ende von RESOURCE.C *
 ***********************/

